SystemOrganization addCategory: #ZTimestamp!

Magnitude subclass: #ZTimestamp
	instanceVariableNames: 'jdn secs'
	classVariableNames: 'ClockEpoch ClockTimezoneOffset UnixEpoch Zero'
	poolDictionaries: ''
	category: 'ZTimestamp'!

!ZTimestamp commentStamp: 'SvenVanCaekenberghe 4/23/2012 10:46' prior: 0!
I am ZTimestamp.
I am a Magnitude.
I represent a point in time, a combination of a date and a time.

I am an alternative for DateAndTime and TimeStamp.
I have second precision and live in the UTC/GMT/Zulu timezone.
I use ISO/International conventions and protocols only. 
I support some essential arithmetic.

I have an efficient internal representation:

	jnd - the julian day number <SmallInteger>
	secs - the number of seconds since midnight, the beginning of the day <SmallInteger>

Examples:

	ZTimestamp now.
	ZTimestamp fromString: '1969-07-20T20:17:40Z'.

There is some compatibility with existing, standard Chronology objects.
I correctly parse representations with a timezone designator
and can print a representation in arbitrary timezones. 

	(ZTimestamp fromString: DateAndTime now truncated printString) localPrintString.!

!ZTimestamp class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/24/2012 17:28'!
clockEpoch
	"Return the epoch of the clock that we use as an instance (cached), see #now"
	
	^ ClockEpoch ifNil: [
		ClockEpoch := self year: 1901 month: 1 day: 1 ]! !

!ZTimestamp class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/24/2012 17:29'!
clockSeconds
	"Return the total seconds count of the clock that we use as an Integer, see #now"

	^ Time totalSeconds! !

!ZTimestamp class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/24/2012 17:29'!
clockTimezoneOffset
	"Return the timezone offset of the clock that we use as an Integer seconds (cached), see #now"

	^ ClockTimezoneOffset ifNil: [
		ClockTimezoneOffset := DateAndTime localTimeZone offset asSeconds ]! !

!ZTimestamp class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/24/2012 17:23'!
epoch
	"Return the epoch of my instances, the point in time from which they start counting, as an instance"
	
	^ self zero! !

!ZTimestamp class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/24/2012 17:30'!
fromClockSeconds: seconds
	"Create and return a new instance from a specific number of seconds since our clock epoch, see #now"
	
	^ self new
		jdn: self clockEpoch julianDayNumber + (seconds // 86400)
		secs: self clockEpoch secondsSinceMidnight + (seconds \\ 86400)
	! !

!ZTimestamp class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 4/21/2012 20:03'!
fromJulianDate: julianDate
	"Create and return a new ZTimestamp instance from the scientific Julian Date, a Float number of Julian Days"
	
	^ self new
		jdn: julianDate truncated
		secs: (julianDate fractionPart * 86400) truncated
		! !

!ZTimestamp class methodsFor: 'ston' stamp: 'SvenVanCaekenberghe 4/28/2012 08:37'!
fromSton: stonReader
	"Read our ISO representation"
	
	^ self readFrom: stonReader parseListSingleton readStream! !

!ZTimestamp class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 4/24/2012 17:27'!
fromString: string
	"Parse and return an instance from a flexible ISO 8601 string representation, see #readFrom:" 

	^ self readFrom: string readStream! !

!ZTimestamp class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 11/1/2012 13:40'!
fromString: string format: formatSpecification
	"Parse and return an instance from the string representation 
	according to formatSpecification, see #readFrom:format:" 

	^ self readFrom: string readStream format: formatSpecification! !

!ZTimestamp class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 4/21/2012 20:04'!
fromUnixTime: integer
	"Create and return a new ZTimestamp from the integer number of seconds since the Unix or POSIX epoch"
	
	^ self unixEpoch + integer seconds
! !

!ZTimestamp class methodsFor: 'class initialization' stamp: 'SvenVanCaekenberghe 9/14/2012 17:02'!
initialize
	Smalltalk addToStartUpList: self.! !

!ZTimestamp class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/21/2012 20:31'!
jdnFromYear: year month: month day: day
	"Compute and return the Julian Day Number from the Gregorian Calender date described by year, month and day.
	See http://en.wikipedia.org/wiki/Julian_date#Converting_Gregorian_calendar_date_to_Julian_Day_Number"
	 
	| a y m |
	a := (14 - month) // 12.
	y := year + 4800 - a.
	m := month + (12 * a) - 3.
	^ day 
		+ (((153 * m) + 2) // 5) 
		+ (365 * y)
		+ (y // 4)
		- (y // 100)
		+ (y // 400)
		- 32045! !

!ZTimestamp class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/24/2012 17:25'!
localTimeZone
	"Return the current machine's local timezone offset as a Duration"
	
	^ DateAndTime localTimeZone! !

!ZTimestamp class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 4/21/2012 20:04'!
now
	"Create and return a new ZTimestamp equal to the current system time"
	
	^ self fromClockSeconds: (self clockSeconds - self clockTimezoneOffset)! !

!ZTimestamp class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 10/31/2012 21:00'!
readFrom: stream
	"Parse a ISO 8601 representation from stream, in the format
	year[-]month[-]day[T]hour[:]minute[:]second[[+|-]hh[:]mm]
	where the whole part after T as well as the TZ offset is optional.
	TZ offset can be Z, all separators can be any non digit"

	| year month day hour minute second offset |
	year := self readInteger: 4 from: stream.
	self readOptionalSeparatorFrom: stream.
	month := self readInteger: 2 from: stream.
	self readOptionalSeparatorFrom: stream.
	day := self readInteger: 2 from: stream.
	self readOptionalSeparatorFrom: stream.
	stream atEnd
		ifFalse: [
			hour := self readInteger: 2 from: stream.
			self readOptionalSeparatorFrom: stream.
			minute := self readInteger: 2 from: stream.
			self readOptionalSeparatorFrom: stream.
			second := self readInteger: 2 from: stream.
			offset := self readTimezoneOffsetFrom: stream ]
		ifTrue: [
			hour := minute := second := offset := 0 ].
	^ self 
		year: year
		month: month
		day: day
		hour: hour
		minute: minute
		second: second
		offset: (offset = 0 ifTrue: [ 0 ] ifFalse: [ offset seconds ])! !

!ZTimestamp class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 11/1/2012 13:43'!
readFrom: stream format: formatSpecification
	"Parse a representation from stream, according to formatSpecification
	and return an instance of me. See ZTimestampFormat for details."

	^ (ZTimestampFormat fromString: formatSpecification)
			readFrom: stream ! !

!ZTimestamp class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/24/2012 17:32'!
readInteger: numberOfDigits from: stream
	"Parse and return a decimal number of numberOfDigits from stream.
	Fail if that is not possible"

	| integer |
	integer := 0.
	numberOfDigits timesRepeat: [ | char |
		char := stream next.
		self assert: ('0123456789' includes: char).
		integer := (integer * 10) + char digitValue ].
	^ integer! !

!ZTimestamp class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/24/2012 17:33'!
readOptionalSeparatorFrom: stream
	"Read an optional separator (non decimal digit) from stream and return it.
	Return nil if nothing was read"

	^ (stream atEnd or: [ '0123456789' includes: stream peek])
		ifTrue: [ nil ]
		ifFalse: [ stream next ]! !

!ZTimestamp class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/24/2012 17:34'!
readTimezoneOffsetFrom: stream
	"Read and return an optional timezone offset in the form of 
	[+|-]hh[separator]mm from stream as an Integer seconds.
	If there is no offset, just return 0"

	| sign hour minute |
	stream atEnd ifTrue: [ ^ 0 ].
	stream peek = $Z ifTrue: [ stream next. ^ 0 ].
	^ ('+-' includes: stream peek)
		ifTrue: [
			sign := stream next = $- ifTrue: [ -1 ] ifFalse: [ 1 ].
			hour := self readInteger: 2 from: stream.
			(self readOptionalSeparatorFrom: stream)
				ifNil: [ minute := 0 ]
				ifNotNil: [ minute := self readInteger: 2 from: stream ].
			sign * (hour * 3600) + (minute * 60) ]
		ifFalse: [ 0 ]
! !

!ZTimestamp class methodsFor: 'system startup' stamp: 'SvenVanCaekenberghe 9/14/2012 17:01'!
startUp
	"Clear the cached clock timezone offset"
	
	ClockTimezoneOffset := nil! !

!ZTimestamp class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/18/2012 10:38'!
unixEpoch
	"Return the Unix or POSIX epoch as a ZTimestamp"
	
	^ UnixEpoch 
		ifNil: [ UnixEpoch := self year: 1970 month: 1 day: 1 ]! !

!ZTimestamp class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/3/2012 19:20'!
withJdn: jdn dayMonthYearDo: block
	"Return the value of executing block with the Gregorian Calender day, month and year as arguments,
	as computed from my Julian Day Number, jdn.
	See http://en.wikipedia.org/wiki/Julian_date#Gregorian_calendar_from_Julian_day_number"
	
	| j g dg c dc b db a da y m d |
	j := jdn + 32044.
	g := j // 146097.
	dg := j \\ 146097.
	c := ((dg // 36524) + 1) * 3 // 4.
	dc := dg - (c * 36524).
	b := dc // 1461.
	db := dc \\ 1461.
	a := ((db // 365) + 1) * 3 // 4.
	da := db - (a * 365).
	y := (g * 400) + (c * 100) + (b * 4) + a.
	m := ((((da * 5) + 308)) // 153) - 2.
	d := da - ((m + 4) * 153 // 5) + 122.
	^ block
		value: d + 1
		value: ((m + 2) \\ 12) + 1
		value: (y - 4800 + ((m + 2) // 12)) ! !

!ZTimestamp class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 4/21/2012 20:06'!
year: year month: month day: day
	"Create and return a new ZTimestamp for the Gregorian calendar date described by year, month and date"
	
	^ self new
		jdn: (self jdnFromYear: year month: month day: day)
		secs: 0! !

!ZTimestamp class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 4/24/2012 17:35'!
year: year month: month day: day hour: hour minute: minute second: second
	"Create and return a new ZTimestamp for the Gregorian calendar date described by year, month and date
	and the Civial time of day on that date described by hour, minute and second"

	^ self new
		jdn: (self jdnFromYear: year month: month day: day)
		secs: (hour * 3600) + (minute * 60) + second! !

!ZTimestamp class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 10/31/2012 20:59'!
year: year month: month day: day hour: hour minute: minute second: second offset: offset
	"Create and return a new ZTimestamp for the Gregorian calendar date described by year, month and date
	and the Civial time of day on that date described by hour, minute and second.
	If offset is non zero, the result is shifted accordingly."

	| timestamp |
	(timestamp := self new)
		jdn: (self jdnFromYear: year month: month day: day)
		secs: (hour * 3600) + (minute * 60) + second.
	offset isZero 
		ifFalse: [ timestamp := timestamp - offset ].
	^ timestamp ! !

!ZTimestamp class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/18/2012 10:38'!
zero
	"Return my zero instance, the point in time from which we start counting, see #epoch"

	^ Zero 
		ifNil: [ 
			Zero :=  self new
				jdn: 0 secs: 0 ]! !

!ZTimestamp methodsFor: 'arithmetic' stamp: 'SvenVanCaekenberghe 4/21/2012 19:54'!
+ operand
	"Return a new ZTimestamp that is the result of adding
	the operand considered as a Duration to the receiver"
	
	^ self class new
		jdn: jdn
		secs: secs + operand asDuration asSeconds! !

!ZTimestamp methodsFor: 'arithmetic' stamp: 'SvenVanCaekenberghe 4/21/2012 19:56'!
- operand
	"For ZTimestamp compatible operands, return a new Duration equal to the difference between the reveiver and the operand.
	Else return a new ZTimestamp that is the result of subtracting the operand considered as a Duration from the receiver"
	
	^ (operand respondsTo: #asZTimestamp)
		ifTrue: [ | timestamp |
			timestamp := operand asZTimestamp.
			Duration seconds:
				(jdn - timestamp julianDayNumber) * 86400
					+ (secs - timestamp secondsSinceMidnight) ]
		ifFalse: [ self + (operand negated) ]! !

!ZTimestamp methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/24/2012 16:42'!
< aZTimestamp
	"Answer true when aZTimestamp is strictly later than me, false otherwise.
	Part of the Magnitude protocol"
	
	^ aZTimestamp julianDayNumber = jdn
		ifTrue: [ secs < aZTimestamp secondsSinceMidnight ]
		ifFalse: [ jdn < aZTimestamp julianDayNumber ]! !

!ZTimestamp methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/24/2012 16:43'!
= aZTimestamp
	"Answer true when aZTimestamp equals the receiver, false otherwise.
	Part of the Magnitude protocol"
	
	self == aZTimestamp ifTrue: [ ^ true ].
	(aZTimestamp isKindOf: self class) ifFalse: [ ^ false ].
	^ aZTimestamp julianDayNumber = jdn
		and: [ aZTimestamp secondsSinceMidnight = secs ]! !

!ZTimestamp methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 7/6/2012 16:52'!
asBeginOfDay
	"Return a timestamp at the begin of the day that I represent"
	
	^ self class new
		jdn: jdn secs: 0 ! !

!ZTimestamp methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 4/24/2012 17:13'!
asDate
	"Convert the receiver to a Date"
	
	^ self asDateAndTime asDate! !

!ZTimestamp methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 6/19/2012 08:57'!
asDateAndTime 
	"Convert the receiver to a DateAndTime.
	Note that I am in the UTC/GMT/Zulu timezone and have second precision"
	
	^ DateAndTime new
		setJdn: jdn 
		seconds: secs 
		nano: 0 
		offset: self offset! !

!ZTimestamp methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 7/6/2012 16:52'!
asEndOfDay
	"Return a timestamp at the end of the day that I represent"
	
	^ self class new
		jdn: jdn secs: 86399! !

!ZTimestamp methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 4/24/2012 17:15'!
asMonth
	"Convert the receiver to a Month"
	
	^ self asDateAndTime asMonth! !

!ZTimestamp methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 4/24/2012 17:16'!
asTime
	"Convert the receiver to a Time in the current day.
	Note that I have second precision"
	
	^ Time seconds: secs! !

!ZTimestamp methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 6/19/2012 08:57'!
asTimeStamp 
	"Convert the receiver to a TimeStamp.
	Note that I am in the UTC/GMT/Zulu timezone and have second precision"

	^ TimeStamp new
		setJdn: jdn 
		seconds: secs 
		nano: 0 
		offset: self offset! !

!ZTimestamp methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 4/21/2012 19:59'!
asUnixTime
	"Return the Integer number of seconds since the Unix or POSIX epoch."
	
	^ (self - self class unixEpoch) asSeconds! !

!ZTimestamp methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 4/24/2012 17:16'!
asWeek
	"Convert the receiver to a Week"
	
	^ self asDateAndTime asWeek! !

!ZTimestamp methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 4/24/2012 17:17'!
asYear
	"Convert the receiver to a Year"
	
	^ self asDateAndTime asYear! !

!ZTimestamp methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 4/24/2012 17:18'!
asZTimestamp
	"Convert the receiver to a ZTimestamp.
	Note that all objects implementing this method will be considered compatible operands for #-"
	
	^ self! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/24/2012 16:57'!
day
	"Return the day of the year of the receiver, according to the Gregorian calendar, a number between 1 and 365"
	
	^ self dayOfYear! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/24/2012 16:58'!
dayMonthYearDo: block
	"Compute and return the Julian Day Number from the Gregorian Calender date described by year, month and day.
	See http://en.wikipedia.org/wiki/Julian_date#Converting_Gregorian_calendar_date_to_Julian_Day_Number"

	^ self class 
		withJdn: jdn dayMonthYearDo: block! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/24/2012 16:59'!
dayOfMonth
	"Return the day of the month of the receiver, according to the Gregorian calendar, a number between 1 and 31"

	^ self dayMonthYearDo: [ :day :month :year | day ]! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/24/2012 17:00'!
dayOfWeek
	"Return the day of the week of the receiver, according to the Gregorian calendar, a number between 1 and 7.
	1 = Sunday, 2 = Monday, .. , 7 = Saterday."

	^ ((jdn + 1) \\ 7) + 1! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/24/2012 17:11'!
dayOfWeekName
	"Return the English name of the day of the week of the receiver, according to the Gregorian calendar, a Symbol"

	^ #(Sunday Monday Tuesday Wednesday Thursday Friday Saterday) at: self dayOfWeek! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/24/2012 17:01'!
dayOfYear
	"Return the day of the year of the receiver, according to the Gregorian calendar, a number between 1 and 365"

	^ self dayMonthYearDo: [ :day :month :year | | monthStart |
		monthStart := #(1 32 60 91 121 152 182 213 244 274 305 335) at: month.
		(month > 2 and: [ Year isLeapYear: year ])
			ifTrue: [ monthStart + day ]
			ifFalse: [ monthStart + day - 1 ] ]! !

!ZTimestamp methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 11/1/2012 13:45'!
format: formatSpecification
	"Return a string representation of me according to formatSpecification.
	See ZTimestampFormat for details."
	
	^ (ZTimestampFormat fromString: formatSpecification)
			format: self! !

!ZTimestamp methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 11/1/2012 13:46'!
format: formatSpecification on: stream
	"Write a string representation of me according to formatSpecification on stream.
	See ZTimestampFormat for details."

	^ (ZTimestampFormat fromString: formatSpecification)
			format: self on: stream! !

!ZTimestamp methodsFor: 'hash' stamp: 'SvenVanCaekenberghe 5/11/2012 07:41'!
hash
	"Return an Integer hash value for the receiver.
	Part of the Magnitude protocl"
	
	^ jdn hashMultiply bitXor: secs! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/24/2012 17:09'!
hour
	"Return the number of hours in the receiver, according to Civilian time, a number between 0 and 23"

	^ (secs \\ 86400) // 3600! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/24/2012 17:09'!
hours
	"Return the number of hours in the receiver, according to Civilian Time, a number between 0 and 23"

	^ self hour! !

!ZTimestamp methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 4/24/2012 16:45'!
jdn: julianDayNumber secs: secondsSinceMidnight
	"Initialize the receiver with julianDayNumber and secondsSinceMidnight,
	then normalize so that secondsSinceMidnight is within the expected range"
	
	jdn := julianDayNumber.
	secs := secondsSinceMidnight.
	self normalize! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/21/2012 20:00'!
julianDate
	"Return the scientific Julian Date, a Float number of Julian Days"
	
	^ jdn + (secs / 86400.0)! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/24/2012 17:05'!
julianDayNumber
	"Return the Julian Day Number of the receiver, an Integer number of days since the epoch"
	
	^ jdn! !

!ZTimestamp methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 4/21/2012 20:09'!
localPrintString
	"Convencience method to convert the receiver in an ISO 8601 representation using the local timezone"

	^ String streamContents: [ :stream |
		self printLocalOn: stream ]! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/24/2012 17:10'!
minute
	"Return the number of minutes in the receiver after full hours have been removed, 
	according to Civilian Time, a number between 0 and 59"

	^ (secs \\ 3600) // 60! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/24/2012 17:10'!
minutes
	"Return the number of minutes in the receiver after full hours have been removed, 
	according to Civilian Time, a number between 0 and 59"

	^ self minute! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/24/2012 16:58'!
month
	"Return the month of the receiver, according to the Gregorian calendar, a number between 1 and 12"

	^ self dayMonthYearDo: [ :day :month :year | month ]! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/24/2012 16:58'!
monthIndex
	"Return the month of the receiver, according to the Gregorian calendar, a number between 1 and 12"

	^ self month! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/24/2012 17:11'!
monthName
	"Return the English name of the month of the receiver, according to the Gregorian calendar, a Symbol"

	^ #(January Februari March April May June July August September October November December) at: self monthIndex! !

!ZTimestamp methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 4/22/2012 18:55'!
normalize
	"Normalize the receiver such that 0 <= secs < 86400"
	 
	(secs between: 0 and: 86399)
		ifFalse: [
			jdn := jdn + (secs // 86400).
			secs := secs \\ 86400 ]! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/24/2012 17:14'!
offset
	"For compatibility: make clear that my timezone offset is zero.
	I am always in the UTC, GMT or Zulu timezone"
	 
	^ Duration zero! !

!ZTimestamp methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 4/21/2012 19:45'!
printLocalOn: stream
	"Print the receiver on stream in ISO 8601 extended format in the local timezone"
	
	self 
		printOn: stream 
		format: #($- $T $: nil) 
		timezone: self class localTimeZone offset! !

!ZTimestamp methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 4/24/2012 17:19'!
printOn: stream
	"Print the receiver on stream in ISO 8601 extended format"
	
	self 
		printOn: stream 
		format: #($- $T $: $Z)! !

!ZTimestamp methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 11/1/2012 13:44'!
printOn: stream format: format
	"Print the receiver on stream using format, following ISO 8601.
	Format defines 4 characters to use:
	#(dateSeparator dateTimeSeparator timeSeparator timeZoneSeparator)
	Each can be nil to skip that separator"
	
	self dayMonthYearDo: [ :day :month :year |
		year printOn: stream base: 10 length: 4 padded: true.
		(format at: 1) ifNotNil: [ :separator | stream nextPut: separator ].
		month printOn: stream base: 10 length: 2 padded: true.
		(format at: 1) ifNotNil: [ :separator | stream nextPut: separator ].
		day printOn: stream base: 10 length: 2 padded: true ].
	(format at: 2) ifNotNil: [ :separator | stream nextPut: separator ].
	self hour printOn: stream base: 10 length: 2 padded: true.
	(format at: 3) ifNotNil: [ :separator | stream nextPut: separator ].
	self minute printOn: stream base: 10 length: 2 padded: true.
	(format at: 3) ifNotNil: [ :separator | stream nextPut: separator ].
	self second printOn: stream base: 10 length: 2 padded: true.
	(format at: 4) ifNotNil: [ :separator | stream nextPut: separator ]! !

!ZTimestamp methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 4/22/2012 11:34'!
printOn: stream format: format timezone: timezone
	"Print an ISO representation in timezone and adding it as [+|-]HH:MM"

	self assert: (format at: 4) isNil.
	(self + timezone asDuration) printOn: stream format: format.
	stream nextPut: (timezone positive ifTrue: [ $+ ] ifFalse: [ $- ]).
	timezone hours abs printOn: stream base: 10 length: 2 padded: true.
	(format at: 3) ifNotNil: [ :separator | stream nextPut: separator ].
	timezone minutes printOn: stream base: 10 length: 2 padded: true! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/24/2012 17:10'!
second
	"Return the number of seconds in the receiver after full hours and minutes have been removed, 
	according to Civilian Time, a number between 0 and 59"

	^ secs \\ 60! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/24/2012 17:10'!
seconds
	"Return the number of seconds in the receiver after full hours and minutes have been removed, 
	according to Civilian Time, a number between 0 and 59"

	^ self second! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/24/2012 17:09'!
secondsSinceMidnight
	"Return the seconds since midnight since the beginning of the Julian Day of the receiver, 
	an Integer between 0 and 86399, inclusive"
	
	^ secs! !

!ZTimestamp methodsFor: 'ston' stamp: 'SvenVanCaekenberghe 4/28/2012 09:30'!
stonOn: stonWriter
	"Use an ISO representation with all details"
	
	stonWriter writeObject: self listSingleton: 
		(String new: 20 streamContents: [ :stream |
			self printOn: stream ])! !

!ZTimestamp methodsFor: 'ston' stamp: 'SvenVanCaekenberghe 4/28/2012 08:36'!
stonProcessSubObjects: block! !

!ZTimestamp methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/24/2012 17:11'!
year
	"Return the year of the receiver, according to the Gregorian calendar, an Integer"

	^ self dayMonthYearDo: [ :day :month :year | year ]! !

Magnitude subclass: #ZTimestampFormat
	instanceVariableNames: 'specification formatter parser constructor weekdayNames monthNames'
	classVariableNames: 'FormatKeys Formats'
	poolDictionaries: ''
	category: 'ZTimestamp'!

!ZTimestampFormat commentStamp: '<historical>' prior: 0!
I am ZTimestampFormat. an implementation of a textual representation for a timestamp, date or time that can be used for formatting or parsing.

You instanciate me by specifying the textual format by example, based on a #reference timetamp. 

Each component of the example representation is numbered from largest to smallest unit, 1=year, 2=month, 3=dayInMonth, 4=hour (16 in 24 hour format), 5=minute and 6=second, as in the ISO representation: 2001-02-03T16:05:06Z which is a Saterday. Example format strings can be found in my class accessing protocol or in the unit tests.

To specifiy a format, you write the reference date so that it matches the representation that you want.

	(ZTimestampFormat fromString: 'SAT, FEB 03 2001 (16:05:06)')
		format: ZTimestamp now.

I can be used for unabiguous, stricter parsing as well.

	(ZTimestampFormat fromString: '02/03/01 (16:05:06)')
		parse: '10/10/10 (12:01:01)'.
		
The list of possible keys and their interpretation #formatSpecifications.
!

!ZTimestampFormat class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/29/2012 16:48'!
ansiC
	^ self fromString: 'Sat Feb _3 16:05:06 2001'! !

!ZTimestampFormat class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/3/2012 19:01'!
formatKeys
	^ FormatKeys ! !

!ZTimestampFormat class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/3/2012 19:00'!
formatSpecifications
	^ #(
		'2001' 		yearFull
		'01' 			yearTwoDigits
		'02'			monthTwoDigits
		'2'				month
		'Februari'	monthNameCapitalized
		'februari'		monthNameLowercased
		'FEBRUARI'	monthNameUppercased
		'Feb'			monthNameAbbreviatedCapitalized
		'feb'			monthNameAbbreviatedLowercased
		'FEB'			monthNameAbbreviatedUppercased
		'3'				day
		'_3'			dayTwoDigitsSpacePadded
		'03'			dayTwoDigits
		'4'				hour12
		'04'			hour12TwoDigits
		'16'			hour24TwoDigits
		'PM'			daypartUppercased
		'pm'			daypartLowercased
		'5'				minute
		'05'			minuteTwoDigits
		'6'				second
		'06'			secondTwoDigits
		'Z'				timeZoneZ
		'+00:00'		timeZone
		'UTC'			timeZoneAbbreviated
		'Saterday'	weekdayNameCapitalized
		'saterday'	weekdayNameLowercased
		'SATERDAY'	weekdayNameUppercased
		'Sat'			weekdayNameAbbreviatedCapitalized
		'sat'			weekdayNameAbbreviatedLowercased
		'SAT'			weekdayNameAbbreviatedUppercased )! !

!ZTimestampFormat class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/3/2012 19:01'!
formats
	^ Formats! !

!ZTimestampFormat class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 10/29/2012 14:40'!
fromString: example
	^ self new
		specification: example;
		yourself
	! !

!ZTimestampFormat class methodsFor: 'class initialization' stamp: 'SvenVanCaekenberghe 11/3/2012 19:01'!
initialize
	Formats := Dictionary newFromPairs: self formatSpecifications.
	FormatKeys := Formats keys sorted: [ :x :y | x size >= y size ].
	! !

!ZTimestampFormat class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/1/2012 20:43'!
kitchen
	^ self fromString: '3:04PM'! !

!ZTimestampFormat class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/1/2012 20:38'!
reference12
	"The ZTimestamp instance used as a reference for specifying a format by example.
	The possible components are numbered from largest to smallest unit."

	^ ZTimestamp year: 2001 month: 2 day: 3 hour: 4 minute: 5 second: 6! !

!ZTimestampFormat class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/1/2012 20:41'!
reference24
	"The ZTimestamp instance used as a reference for specifying a format by example.
	The possible components are numbered from largest to smallest unit."

	^ ZTimestamp year: 2001 month: 2 day: 3 hour: 16 minute: 5 second: 6! !

!ZTimestampFormat class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/29/2012 16:54'!
rfc3339
	^ self fromString: '2001-02-03T16:05:06Z'! !

!ZTimestampFormat class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/29/2012 16:49'!
rfc822
	^ self fromString: '03 Feb 01 16:05 UTC'! !

!ZTimestampFormat class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/29/2012 16:49'!
rfc850
	^ self fromString: 'Saterday, 03-Feb-01 16:05:06 UTC'! !

!ZTimestampFormat class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/29/2012 16:49'!
unixDate
	^ self fromString: 'Sat Feb _3 16:05:06 UTC 2001'! !

!ZTimestampFormat class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/3/2012 20:19'!
verbose
	^ self fromString: 'Saterday, 03-Februari-2001 04:05:06 PM (+00:00)'! !

!ZTimestampFormat methodsFor: 'private' stamp: 'SvenVanCaekenberghe 11/3/2012 19:11'!
compileSpecification
	| offset elements |
	elements := Array streamContents: [ :stream |
		offset := 1.
		[ offset <= specification size ] whileTrue: [ 
			(self nextFormatKeyStartingAt: offset)
				ifNotNil: [ :formatKey |
					stream nextPut: (self class formats at: formatKey).
					offset := offset + formatKey size ]
				ifNil: [ 
					stream nextPut: (specification at: offset).
					offset := offset + 1 ] ] ].
	formatter := elements collect: [ :each | 
			each isCharacter ifTrue: [ each ] ifFalse: [ (#format:, each, #On:) asSymbol ] ].
	parser := elements collect: [ :each | 
			each isCharacter ifTrue: [ each ] ifFalse: [ (#parse, each capitalized, #From:) asSymbol ] ] ! !

!ZTimestampFormat methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/1/2012 10:40'!
constructor
	^ constructor! !

!ZTimestampFormat methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/1/2012 10:27'!
constructor: anObject
	constructor := anObject! !

!ZTimestampFormat methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 11/1/2012 10:31'!
createDate
	self constructor: [ :elements | 
		Date 
			year: (elements at: #year ifAbsent: [ 0 ]) 
			month: (elements at: #month ifAbsent: [ 0 ])  
			day: (elements at: #day ifAbsent: [ 0 ]) ]! !

!ZTimestampFormat methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 11/3/2012 20:32'!
createDateAndTime
	self constructor: [ :elements | 
		DateAndTime 
			year: (elements at: #year ifAbsent: [ 0 ]) 
			month: (elements at: #month ifAbsent: [ 0 ])  
			day: (elements at: #day ifAbsent: [ 0 ])  
			hour: (elements at: #hour ifAbsent: [ 0 ]) 
			minute: (elements at: #minute ifAbsent: [ 0 ])  
			second: (elements at: #second ifAbsent: [ 0 ]) 
			offset: (elements at: #offset ifAbsent: [ Duration zero ]) ]! !

!ZTimestampFormat methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 11/3/2012 20:19'!
createInstanceFrom: elements
	elements 
		at: #daypart 
		ifPresent: [ :daypart | | hour |
			hour :=  elements at: #hour.
			(daypart = #pm and: [ hour < 12 ])
				ifTrue: [ elements at: #hour put: hour + 12 ].
			(daypart = #am and: [ hour = 12 ])
				ifTrue: [ elements at: #hour put: 0 ] ].
	^ self constructor value: elements! !

!ZTimestampFormat methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 11/1/2012 10:31'!
createTime
	self constructor: [ :elements | 
		Time 
			hour: (elements at: #hour ifAbsent: [ 0 ]) 
			minute: (elements at: #minute ifAbsent: [ 0 ])  
			second: (elements at: #second ifAbsent: [ 0 ]) ]! !

!ZTimestampFormat methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 11/1/2012 10:40'!
createZTimestamp
	self constructor: [ :elements | 
		ZTimestamp 
			year: (elements at: #year ifAbsent: [ 0 ]) 
			month: (elements at: #month ifAbsent: [ 0 ])  
			day: (elements at: #day ifAbsent: [ 0 ])  
			hour: (elements at: #hour ifAbsent: [ 0 ]) 
			minute: (elements at: #minute ifAbsent: [ 0 ])  
			second: (elements at: #second ifAbsent: [ 0 ]) 
			offset: (elements at: #offset ifAbsent: [ 0 ]) ]! !

!ZTimestampFormat methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 11/1/2012 10:34'!
dutch
	self monthNames: #( 
		'Januari' 'Februari' 'Maart' 'April' 'Mei' 'Juni' 
		'Juli' 'Augustus' 'September' 'Oktober' 'November' 'December' ).
	self weekdayNames: #( 
		'Zondag' 'Maandag' 'Dinsdag' 'Woensdag' 'Donderdag' 'Vrijdag' 'Zaterdag' )! !

!ZTimestampFormat methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 11/1/2012 10:39'!
english
	self monthNames: #( 
		January Februari March April May June 
		July August September October November December).
	self weekdayNames: #( 
		Sunday Monday Tuesday Wednesday Thursday Friday Saterday )! !

!ZTimestampFormat methodsFor: 'formatting' stamp: 'SvenVanCaekenberghe 10/29/2012 21:36'!
format: object
	^ String streamContents: [ :stream |
			self print: object on: stream ]! !

!ZTimestampFormat methodsFor: 'formatters' stamp: 'SvenVanCaekenberghe 11/3/2012 20:26'!
format: object dayOn: stream
	object dayOfMonth printOn: stream! !

!ZTimestampFormat methodsFor: 'formatters' stamp: 'SvenVanCaekenberghe 11/3/2012 20:26'!
format: object dayTwoDigitsOn: stream
	object dayOfMonth printOn: stream base: 10 length: 2 padded: true! !

!ZTimestampFormat methodsFor: 'formatters' stamp: 'SvenVanCaekenberghe 11/3/2012 20:18'!
format: object dayTwoDigitsSpacePaddedOn: stream
	object dayOfMonth < 10
		ifTrue: [ stream space ].
	object dayOfMonth printOn: stream! !

!ZTimestampFormat methodsFor: 'formatters' stamp: 'SvenVanCaekenberghe 11/3/2012 20:16'!
format: object daypartLowercasedOn: stream
	stream << (object hour < 12 ifTrue: [ #am ] ifFalse: [ #pm ])! !

!ZTimestampFormat methodsFor: 'formatters' stamp: 'SvenVanCaekenberghe 11/3/2012 20:16'!
format: object daypartUppercasedOn: stream
	stream << (object hour < 12 ifTrue: [ #AM ] ifFalse: [ #PM ])! !

!ZTimestampFormat methodsFor: 'formatters' stamp: 'SvenVanCaekenberghe 11/3/2012 20:13'!
format: object hour12On: stream
	| hour |
	hour := object hour.
	(hour > 12
		ifTrue: [ hour - 12 ]
		ifFalse: [ 
			hour < 1
				ifTrue: [ 12 ]
				ifFalse: [ hour ] ]) printOn: stream! !

!ZTimestampFormat methodsFor: 'formatters' stamp: 'SvenVanCaekenberghe 11/3/2012 20:14'!
format: object hour12TwoDigitsOn: stream
	| hour |
	hour := object hour.
	(hour > 12
		ifTrue: [ hour - 12 ]
		ifFalse: [ 
			hour < 1
				ifTrue: [ 12 ]
				ifFalse: [ hour ] ]) printOn: stream base: 10 length: 2 padded: true! !

!ZTimestampFormat methodsFor: 'formatters' stamp: 'SvenVanCaekenberghe 11/3/2012 20:15'!
format: object hour24TwoDigitsOn: stream
	object hour printOn: stream base: 10 length: 2 padded: true! !

!ZTimestampFormat methodsFor: 'formatters' stamp: 'SvenVanCaekenberghe 11/3/2012 20:10'!
format: object minuteOn: stream
	object minute printOn: stream! !

!ZTimestampFormat methodsFor: 'formatters' stamp: 'SvenVanCaekenberghe 11/3/2012 20:10'!
format: object minuteTwoDigitsOn: stream
	object minute printOn: stream base: 10 length: 2 padded: true! !

!ZTimestampFormat methodsFor: 'formatters' stamp: 'SvenVanCaekenberghe 11/3/2012 20:08'!
format: object monthNameAbbreviatedCapitalizedOn: stream
	stream << ((self monthNames at: object month) copyFrom: 1 to: 3)! !

!ZTimestampFormat methodsFor: 'formatters' stamp: 'SvenVanCaekenberghe 11/3/2012 20:08'!
format: object monthNameAbbreviatedLowercasedOn: stream
	stream << ((self monthNames at: object month) copyFrom: 1 to: 3) asLowercase ! !

!ZTimestampFormat methodsFor: 'formatters' stamp: 'SvenVanCaekenberghe 11/3/2012 20:09'!
format: object monthNameAbbreviatedUppercasedOn: stream
	stream << ((self monthNames at: object month) copyFrom: 1 to: 3) asUppercase ! !

!ZTimestampFormat methodsFor: 'formatters' stamp: 'SvenVanCaekenberghe 11/3/2012 20:09'!
format: object monthNameCapitalizedOn: stream
	stream << (self monthNames at: object month)! !

!ZTimestampFormat methodsFor: 'formatters' stamp: 'SvenVanCaekenberghe 11/3/2012 20:09'!
format: object monthNameLowercasedOn: stream
	stream << (self monthNames at: object month) asLowercase ! !

!ZTimestampFormat methodsFor: 'formatters' stamp: 'SvenVanCaekenberghe 11/3/2012 20:10'!
format: object monthNameUppercasedOn: stream
	stream << (self monthNames at: object month) asUppercase ! !

!ZTimestampFormat methodsFor: 'formatters' stamp: 'SvenVanCaekenberghe 11/3/2012 20:07'!
format: object monthOn: stream
	object month printOn: stream! !

!ZTimestampFormat methodsFor: 'formatters' stamp: 'SvenVanCaekenberghe 11/3/2012 20:07'!
format: object monthTwoDigitsOn: stream
	object month printOn: stream base: 10 length: 2 padded: true! !

!ZTimestampFormat methodsFor: 'formatters' stamp: 'SvenVanCaekenberghe 11/3/2012 20:06'!
format: object secondOn: stream
	object second printOn: stream! !

!ZTimestampFormat methodsFor: 'formatters' stamp: 'SvenVanCaekenberghe 11/3/2012 20:06'!
format: object secondTwoDigitsOn: stream
	object second printOn: stream base: 10 length: 2 padded: true! !

!ZTimestampFormat methodsFor: 'formatters' stamp: 'SvenVanCaekenberghe 11/3/2012 20:05'!
format: object timeZoneAbbreviatedOn: stream
	object offset isZero
		ifTrue: [ stream << #UTC ]
		ifFalse: [ stream << (TimeZone offset: object offset) abbreviation ]! !

!ZTimestampFormat methodsFor: 'formatters' stamp: 'SvenVanCaekenberghe 11/3/2012 20:05'!
format: object timeZoneOn: stream
	stream nextPut: (object offset positive ifTrue: [ $+ ] ifFalse: [ $- ]).
	object offset hours abs
		printOn: stream base: 10 length: 2 padded: true.
	stream nextPut: $:.
	object offset minutes
		printOn: stream base: 10 length: 2 padded: true ! !

!ZTimestampFormat methodsFor: 'formatters' stamp: 'SvenVanCaekenberghe 11/3/2012 20:05'!
format: object timeZoneZOn: stream
	object offset isZero
		ifTrue: [ stream nextPut: $Z ]
		ifFalse: [ 
			stream nextPut: (object offset positive ifTrue: [ $+ ] ifFalse: [ $- ]).
			object offset hours abs
				printOn: stream base: 10 length: 2 padded: true.
			stream nextPut: $:.
			object offset minutes
				printOn: stream base: 10 length: 2 padded: true ]! !

!ZTimestampFormat methodsFor: 'formatters' stamp: 'SvenVanCaekenberghe 11/3/2012 19:52'!
format: object weekdayNameAbbreviatedCapitalizedOn: stream
	stream << ((self weekdayNames at: object dayOfWeek) copyFrom: 1 to: 3)! !

!ZTimestampFormat methodsFor: 'formatters' stamp: 'SvenVanCaekenberghe 11/3/2012 19:52'!
format: object weekdayNameAbbreviatedLowercasedOn: stream
	stream << ((self weekdayNames at: object dayOfWeek) copyFrom: 1 to: 3) asLowercase ! !

!ZTimestampFormat methodsFor: 'formatters' stamp: 'SvenVanCaekenberghe 11/3/2012 19:52'!
format: object weekdayNameAbbreviatedUppercasedOn: stream
	stream << ((self weekdayNames at: object dayOfWeek) copyFrom: 1 to: 3) asUppercase ! !

!ZTimestampFormat methodsFor: 'formatters' stamp: 'SvenVanCaekenberghe 11/3/2012 19:51'!
format: object weekdayNameCapitalizedOn: stream
	stream << (self weekdayNames at: object dayOfWeek)! !

!ZTimestampFormat methodsFor: 'formatters' stamp: 'SvenVanCaekenberghe 11/3/2012 19:51'!
format: object weekdayNameLowercasedOn: stream
	stream << (self weekdayNames at: object dayOfWeek) asLowercase ! !

!ZTimestampFormat methodsFor: 'formatters' stamp: 'SvenVanCaekenberghe 11/3/2012 19:51'!
format: object weekdayNameUppercasedOn: stream
	stream << (self weekdayNames at: object dayOfWeek) asUppercase ! !

!ZTimestampFormat methodsFor: 'formatters' stamp: 'SvenVanCaekenberghe 11/3/2012 19:43'!
format: object yearFullOn: stream
	object year printOn: stream base: 10 length: 4 padded: true! !

!ZTimestampFormat methodsFor: 'formatters' stamp: 'SvenVanCaekenberghe 11/3/2012 19:43'!
format: object yearTwoDigitsOn: stream
	object year \\ 100 printOn: stream base: 10 length: 2 padded: true! !

!ZTimestampFormat methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/3/2012 19:05'!
formatter
	^ formatter! !

!ZTimestampFormat methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 11/1/2012 10:35'!
french
	self monthNames: #( 
		'Janvier' 'Fvrier' 'Mars' 'Avril' 'Mai' 'Juin' 
		'Juillet' 'Aot' 'Septembre' 'Octobre' 'Novembre' 'Dcembre' ).
	self weekdayNames: #( 
		'Dimanche' 'Lundi' 'Mardi' 'Mecredi' 'Jeudi' 'Vendredi' 'Samedi' )! !

!ZTimestampFormat methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 11/1/2012 10:35'!
german
	self monthNames: #( 
		'Januar' 'Februar' 'Mrz' 'April' 'Mai' 'Juni' 
		'Juli' 'August' 'September' 'Oktober' 'November' 'Dezember' ).
	self weekdayNames: #( 
		'Sonntag' 'Montag' 'Dienstag' 'Mittwoch' 'Donnerstag' 'Freitag' 'Samstag' )! !

!ZTimestampFormat methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 11/1/2012 10:40'!
initialize 
	super initialize.
	self english; createZTimestamp! !

!ZTimestampFormat methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/1/2012 10:41'!
monthNames
	^ monthNames! !

!ZTimestampFormat methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/29/2012 13:52'!
monthNames: anObject
	monthNames := anObject! !

!ZTimestampFormat methodsFor: 'private' stamp: 'SvenVanCaekenberghe 11/3/2012 20:12'!
nextFormatKeyStartingAt: offset
	^ self class formatKeys
		detect: [ :each | self string: specification contains: each startingAt: offset ]
		ifNone: [ nil ]! !

!ZTimestampFormat methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 10/30/2012 10:28'!
parse: string
	^ self readFrom: string readStream! !

!ZTimestampFormat methodsFor: 'parsers' stamp: 'SvenVanCaekenberghe 11/3/2012 19:41'!
parseDayFrom: stream
	^ #day -> (self readInteger: 1 atLeastFrom: stream)! !

!ZTimestampFormat methodsFor: 'parsers' stamp: 'SvenVanCaekenberghe 11/3/2012 19:42'!
parseDayTwoDigitsFrom: stream
	^ #day -> (self readInteger: 2 from: stream)! !

!ZTimestampFormat methodsFor: 'parsers' stamp: 'SvenVanCaekenberghe 11/3/2012 19:42'!
parseDayTwoDigitsSpacePaddedFrom: stream
	^ #day -> (self readInteger: 2 optionalSpaceFrom: stream)! !

!ZTimestampFormat methodsFor: 'parsers' stamp: 'SvenVanCaekenberghe 11/3/2012 19:40'!
parseDaypartLowercasedFrom: stream
	| part |
	part := stream next: 2.
	self assert: (#(#am #pm) includes: part).
	^ #daypart -> part asSymbol! !

!ZTimestampFormat methodsFor: 'parsers' stamp: 'SvenVanCaekenberghe 11/3/2012 19:39'!
parseDaypartUppercasedFrom: stream
	| part |
	part := stream next: 2.
	self assert: (#(#AM #PM) includes: part).
	^ #daypart -> part asLowercase asSymbol! !

!ZTimestampFormat methodsFor: 'parsers' stamp: 'SvenVanCaekenberghe 11/3/2012 19:38'!
parseHour12From: stream
	^ #hour -> (self readInteger: 1 atLeastFrom: stream)! !

!ZTimestampFormat methodsFor: 'parsers' stamp: 'SvenVanCaekenberghe 11/3/2012 19:38'!
parseHour12TwoDigitsFrom: stream
	^ #hour -> (self readInteger: 2 from: stream)! !

!ZTimestampFormat methodsFor: 'parsers' stamp: 'SvenVanCaekenberghe 11/3/2012 19:38'!
parseHour24TwoDigitsFrom: stream
	^ #hour -> (self readInteger: 2 from: stream)! !

!ZTimestampFormat methodsFor: 'parsers' stamp: 'SvenVanCaekenberghe 11/3/2012 19:36'!
parseMinuteFrom: stream
	^ #minute -> (self readInteger: 1 atLeastFrom: stream)! !

!ZTimestampFormat methodsFor: 'parsers' stamp: 'SvenVanCaekenberghe 11/3/2012 20:30'!
parseMinuteTwoDigitsFrom: stream
	^ #minute -> (self readInteger: 2 from: stream)! !

!ZTimestampFormat methodsFor: 'parsers' stamp: 'SvenVanCaekenberghe 11/3/2012 19:36'!
parseMonthFrom: stream
	^ #month -> (self readInteger: 1 atLeastFrom: stream)! !

!ZTimestampFormat methodsFor: 'parsers' stamp: 'SvenVanCaekenberghe 11/3/2012 20:28'!
parseMonthNameAbbreviatedCapitalizedFrom: stream
	| abbreviation fullName target |
	abbreviation := stream next: 3.
	target := abbreviation asLowercase capitalized.
	fullName := self monthNames detect: [ :each | 
		self string: each contains: target startingAt: 1 ].
	self assert: (fullName copyFrom: 1 to: 3) capitalized = abbreviation.
	^ #month -> (self monthNames indexOf: fullName)! !

!ZTimestampFormat methodsFor: 'parsers' stamp: 'SvenVanCaekenberghe 11/3/2012 20:28'!
parseMonthNameAbbreviatedLowercasedFrom: stream
	| abbreviation fullName target |
	abbreviation := stream next: 3.
	target := abbreviation asLowercase capitalized.
	fullName := self monthNames detect: [ :each | 
		self string: each contains: target startingAt: 1 ].
	self assert: (fullName copyFrom: 1 to: 3) asLowercase = abbreviation.
	^ #month -> (self monthNames indexOf: fullName)! !

!ZTimestampFormat methodsFor: 'parsers' stamp: 'SvenVanCaekenberghe 11/3/2012 20:28'!
parseMonthNameAbbreviatedUppercasedFrom: stream
	| abbreviation fullName target |
	abbreviation := stream next: 3.
	target := abbreviation asLowercase capitalized.
	fullName := self monthNames detect: [ :each | 
		self string: each contains: target startingAt: 1 ].
	self assert: (fullName copyFrom: 1 to: 3) asUppercase = abbreviation.
	^ #month -> (self monthNames indexOf: fullName)! !

!ZTimestampFormat methodsFor: 'parsers' stamp: 'SvenVanCaekenberghe 11/3/2012 20:28'!
parseMonthNameCapitalizedFrom: stream
	| match |
	match := self readAny: self monthNames sorted from: stream.
	self assert: match notNil.
	^ #month -> (self monthNames indexOf: match)! !

!ZTimestampFormat methodsFor: 'parsers' stamp: 'SvenVanCaekenberghe 11/3/2012 20:28'!
parseMonthNameLowercasedFrom: stream
	| match |
	match := self 
		readAny: (self monthNames collect: [ :each | each asLowercase ]) sorted 
		from: stream.
	self assert: match notNil.
	^ #month -> (self monthNames indexOf: match capitalized)! !

!ZTimestampFormat methodsFor: 'parsers' stamp: 'SvenVanCaekenberghe 11/3/2012 20:28'!
parseMonthNameUppercasedFrom: stream
	| match |
	match := self 
		readAny: (self monthNames collect: [ :each | each asUppercase ]) sorted 
		from: stream.
	self assert: match notNil.
	^ #month -> (self monthNames indexOf: match asLowercase capitalized)! !

!ZTimestampFormat methodsFor: 'parsers' stamp: 'SvenVanCaekenberghe 11/3/2012 20:27'!
parseMonthTwoDigitsFrom: stream
	^ #month -> (self readInteger: 2 from: stream)! !

!ZTimestampFormat methodsFor: 'parsers' stamp: 'SvenVanCaekenberghe 11/3/2012 19:35'!
parseSecondFrom: stream
	^ #second -> (self readInteger: 1 atLeastFrom: stream)! !

!ZTimestampFormat methodsFor: 'parsers' stamp: 'SvenVanCaekenberghe 11/3/2012 20:30'!
parseSecondTwoDigitsFrom: stream
	^ #second -> (self readInteger: 2 from: stream)! !

!ZTimestampFormat methodsFor: 'parsers' stamp: 'SvenVanCaekenberghe 11/3/2012 19:33'!
parseTimeZoneAbbreviatedFrom: stream
	^ #offset -> (TimeZone abbreviated: (stream next: 3)) offset! !

!ZTimestampFormat methodsFor: 'parsers' stamp: 'SvenVanCaekenberghe 11/3/2012 19:33'!
parseTimeZoneFrom: stream
	^ #offset -> (ZTimestamp readTimezoneOffsetFrom: stream) seconds! !

!ZTimestampFormat methodsFor: 'parsers' stamp: 'SvenVanCaekenberghe 11/3/2012 19:32'!
parseTimeZoneZFrom: stream
	^ #offset -> (ZTimestamp readTimezoneOffsetFrom: stream) seconds! !

!ZTimestampFormat methodsFor: 'parsers' stamp: 'SvenVanCaekenberghe 11/3/2012 20:28'!
parseWeekdayNameAbbreviatedCapitalizedFrom: stream
	| abbreviation fullName target |
	abbreviation := stream next: 3.
	target := abbreviation asLowercase capitalized.
	fullName := self weekdayNames detect: [ :each | 
		self string: each contains: target startingAt: 1 ].
	self assert: (fullName copyFrom: 1 to: 3) capitalized = abbreviation.
	^ #weekday -> (self weekdayNames indexOf: fullName)! !

!ZTimestampFormat methodsFor: 'parsers' stamp: 'SvenVanCaekenberghe 11/3/2012 20:28'!
parseWeekdayNameAbbreviatedLowercasedFrom: stream
	| abbreviation fullName target |
	abbreviation := stream next: 3.
	target := abbreviation asLowercase capitalized.
	fullName := self weekdayNames detect: [ :each | 
		self string: each contains: target startingAt: 1 ].
	self assert: (fullName copyFrom: 1 to: 3) asLowercase = abbreviation.
	^ #weekday -> (self weekdayNames indexOf: fullName)! !

!ZTimestampFormat methodsFor: 'parsers' stamp: 'SvenVanCaekenberghe 11/3/2012 20:28'!
parseWeekdayNameAbbreviatedUppercasedFrom: stream
	| abbreviation fullName target |
	abbreviation := stream next: 3.
	target := abbreviation asLowercase capitalized.
	fullName := self weekdayNames detect: [ :each | 
		self string: each contains: target startingAt: 1 ].
	self assert: (fullName copyFrom: 1 to: 3) asUppercase = abbreviation.
	^ #weekday -> (self weekdayNames indexOf: fullName)! !

!ZTimestampFormat methodsFor: 'parsers' stamp: 'SvenVanCaekenberghe 11/3/2012 20:29'!
parseWeekdayNameCapitalizedFrom: stream
	| match |
	match := self readAny: self weekdayNames sorted from: stream.
	self assert: match notNil.
	^ #weekday -> (self weekdayNames indexOf: match)! !

!ZTimestampFormat methodsFor: 'parsers' stamp: 'SvenVanCaekenberghe 11/3/2012 20:29'!
parseWeekdayNameLowercasedFrom: stream
	| match |
	match := self 
		readAny: (self weekdayNames collect: [ :each | each asLowercase ]) sorted 
		from: stream.
	self assert: match notNil.
	^ #weekday -> (self weekdayNames indexOf: match capitalized)! !

!ZTimestampFormat methodsFor: 'parsers' stamp: 'SvenVanCaekenberghe 11/3/2012 20:29'!
parseWeekdayNameUppercasedFrom: stream
	| match |
	match := self 
		readAny: (self weekdayNames collect: [ :each | each asUppercase ]) sorted 
		from: stream.
	self assert: match notNil.
	^ #weekday -> (self weekdayNames indexOf: match asLowercase capitalized)! !

!ZTimestampFormat methodsFor: 'parsers' stamp: 'SvenVanCaekenberghe 11/3/2012 19:31'!
parseYearFullFrom: stream
	| sign |
	sign := 1.
	stream peek = $-
		ifTrue: [ 
			stream next.
			sign := -1 ].
	^ #year -> (sign * (self readInteger: 4 atLeastFrom: stream))! !

!ZTimestampFormat methodsFor: 'parsers' stamp: 'SvenVanCaekenberghe 11/3/2012 19:31'!
parseYearTwoDigitsFrom: stream
	^ #year -> (2000 + (self readInteger: 2 from: stream))! !

!ZTimestampFormat methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/3/2012 19:05'!
parser
	^ parser! !

!ZTimestampFormat methodsFor: 'formatting' stamp: 'SvenVanCaekenberghe 11/3/2012 20:23'!
print: object on: stream
	formatter do: [ :each |
		each isCharacter 
			ifTrue: [ stream nextPut: each ] 
			ifFalse: [ self perform: each with: object with: stream ] ]! !

!ZTimestampFormat methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 11/3/2012 20:51'!
printOn: stream
	super printOn: stream.
	stream nextPut: $(.
	stream << specification.
	stream nextPut: $)! !

!ZTimestampFormat methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/30/2012 13:13'!
readAny: sortedStings from: stream
	| candidateIndex matchCount |
	candidateIndex := 1.
	matchCount := 0.
	[
		(sortedStings at: candidateIndex) size = matchCount 
			ifTrue: [ ^ sortedStings at: candidateIndex ].
		(stream atEnd or: [ candidateIndex > sortedStings size ])
			ifTrue: [ ^ nil ].
		(((sortedStings at: candidateIndex) at: matchCount + 1) = stream peek)
			ifTrue: [
				stream next.
				matchCount := matchCount + 1 ]
			ifFalse: [
				candidateIndex := candidateIndex + 1.
				(candidateIndex <= sortedStings size
					and: [ (1 to: matchCount) allSatisfy: [ :each | 
								(sortedStings at: candidateIndex - 1) = (sortedStings at: candidateIndex) ] ])
					ifFalse: [ ^ nil ] ]
	] repeat! !

!ZTimestampFormat methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 11/3/2012 20:21'!
readFrom: stream
	| offset elements |
	elements := SmallDictionary new.
	parser do: [ :each |
		each isCharacter 
			ifTrue: [ self assert: stream next = each ]
			ifFalse: [ elements add: (self perform: each with: stream) ] ].
	^ self createInstanceFrom: elements! !

!ZTimestampFormat methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/30/2012 14:31'!
readInteger: numberOfDigits atLeastFrom: stream
	"Parse and return a decimal number of at least numberOfDigits from stream.
	Continue reading until a non digit or the end of stream is reached.
	Fail if that is not possible"

	| integer |
	integer := 0.
	numberOfDigits timesRepeat: [ | char |
		char := stream next.
		self assert: ('0123456789' includes: char).
		integer := (integer * 10) + char digitValue ].
	[ stream atEnd not and: [ stream peek isDigit ] ] whileTrue: [
		integer := (integer * 10) + stream next digitValue ].
	^ integer! !

!ZTimestampFormat methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/29/2012 21:39'!
readInteger: numberOfDigits from: stream
	"Parse and return a decimal number of numberOfDigits from stream.
	Fail if that is not possible"

	| integer |
	integer := 0.
	numberOfDigits timesRepeat: [ | char |
		char := stream next.
		self assert: ('0123456789' includes: char).
		integer := (integer * 10) + char digitValue ].
	^ integer! !

!ZTimestampFormat methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/30/2012 14:33'!
readInteger: numberOfDigits optionalSpaceFrom: stream
	"Parse and return a decimal number of numberOfDigits from stream.
	There can be one optional space instead of a digit at the beginning.
	Fail if that is not possible"

	^ stream peek = $ 
		ifTrue: [ 
			stream next. 
			self readInteger: numberOfDigits - 1 from: stream ]
		ifFalse: [
			self readInteger: numberOfDigits from: stream ]! !

!ZTimestampFormat methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/3/2012 20:46'!
specification
	^ specification ! !

!ZTimestampFormat methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 11/3/2012 19:04'!
specification: string
	specification := string.
	self compileSpecification! !

!ZTimestampFormat methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/29/2012 15:37'!
string: string contains: subString startingAt: offset
	subString doWithIndex: [ :each :index |
		offset + index - 1 <= string size 
			ifFalse: [ ^ false ].
		(string at: offset + index - 1) = each
			ifFalse: [ ^ false ] ].
	^ true! !

!ZTimestampFormat methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/1/2012 10:41'!
weekdayNames
	^ weekdayNames! !

!ZTimestampFormat methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/29/2012 13:52'!
weekdayNames: anObject
	weekdayNames := anObject! !

!DateAndTime methodsFor: '*ztimestamp' stamp: 'SvenVanCaekenberghe 9/1/2012 20:50'!
asZTimestamp
	^ ZTimestamp new
		jdn: self julianDayNumber 
		secs: seconds - offset asSeconds! !

TestCase subclass: #ZTimestampFormatTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ZTimestamp'!

!ZTimestampFormatTests commentStamp: '<historical>' prior: 0!
I am ZTimestampFormatTests a suite of unit tests for ZTimestampFormat.!

!ZTimestampFormatTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 11/1/2012 19:19'!
testAMPM
	| times format12 format24 |
	times := #( 
		('12:00AM' '00:00') ('12:01AM' '00:01') ('12:59AM' '00:59') ('01:00AM' '01:00') ('11:00AM' '11:00') ('11:59AM' '11:59')
		('12:00PM' '12:00') ('12:01PM' '12:01') ('12:59PM' '12:59')('01:00PM' '13:00') ('11:00PM' '23:00') ('11:59PM' '23:59') ).
	format12 := ZTimestampFormat fromString: '04:05PM'.
	format24 := ZTimestampFormat fromString: '16:05'.
	times do: [ :each |
		self assert: (format12 parse: each first) equals: (format24 parse: each second).
		self assert: (format12 format: (format12 parse: each first)) equals: each first.
		self assert: (format24 format: (format24 parse: each second)) equals: each second ]! !

!ZTimestampFormatTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 11/3/2012 20:32'!
testDateAndTime
	| now format |
	now := DateAndTime now rounded.
	format := ZTimestampFormat fromString: '2001-02-03T16:05:06+00:00'.
	self 
		assert: (format format: now)
		equals: now printString.	
	format createDateAndTime.
	self
		assert: (format parse: now printString)
		equals: now.! !

!ZTimestampFormatTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 11/1/2012 10:44'!
testDateCreation
	self 
		assert: ((ZTimestampFormat fromString: '03/02/2001') createDate; parse: '07/08/1967')
		equals: (Date year: 1967 month: 8 day: 7).	! !

!ZTimestampFormatTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 10/29/2012 16:51'!
testDateFormatting
	self
		assert: ((ZTimestampFormat fromString: '2001/02/03') format: (ZTimestamp year: 2012 month: 10 day: 20))
		equals: '2012/10/20'.
	self
		assert: ((ZTimestampFormat fromString: '3 Februari 2001') format: (ZTimestamp year: 2012 month: 10 day: 20))
		equals: '20 October 2012'.
	self
		assert: ((ZTimestampFormat fromString: '_3 Februari 2001') 
						french;
						format: (ZTimestamp year: 1967 month: 8 day: 7))
		equals: ' 7 Aot 1967'.
	self
		assert: ((ZTimestampFormat fromString: '3 FEB ''01') format: (ZTimestamp year: 2012 month: 10 day: 20))
		equals: '20 OCT ''12'.! !

!ZTimestampFormatTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 10/30/2012 13:20'!
testDateParsing
	self
		assert: ((ZTimestampFormat fromString: '2001/02/03') parse: '2012/10/20') 
		equals: (ZTimestamp year: 2012 month: 10 day: 20).
	self
		assert: ((ZTimestampFormat fromString: '3 FEB ''01') parse: '20 OCT ''12') 
		equals: (ZTimestamp year: 2012 month: 10 day: 20).
	self
		assert: ((ZTimestampFormat fromString: '3 Februari 2001') parse: '20 October 2012') 
		equals: (ZTimestamp year: 2012 month: 10 day: 20).
	self
		assert: ((ZTimestampFormat fromString: '_3 Februari 2001') 
						french;
						parse: ' 7 Aot 1967') 
		equals: (ZTimestamp year: 1967 month: 8 day: 7).! !

!ZTimestampFormatTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 11/3/2012 20:52'!
testPredefinedFormats
	| reference now |
	reference := ZTimestampFormat reference24.
	now := DateAndTime now truncated.
	#(ansiC verbose kitchen rfc3339 rfc822 rfc850 unixDate) 
		do: [ :each | | format target |
			format := ZTimestampFormat perform: each.
			target := format specification copy replaceAll: $_ with: $ . 
			self assert: (format format: reference) equals: target ].
	#(verbose rfc3339 rfc850 unixDate) 
		do: [ :each | | format |
			(format := ZTimestampFormat perform: each)
				createDateAndTime.
			self assert: (format parse: (format format: now)) equals: now ]! !

!ZTimestampFormatTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 11/3/2012 20:32'!
testReferenceDateAndTime
	| reference format now |
	reference := ZTimestampFormat reference12 asDateAndTime.
	(format := ZTimestampFormat fromString: reference printString)
		createDateAndTime.
	self assert: (format format: reference) equals: reference printString.
	self assert: (format parse: reference printString) equals: reference.
	reference := ZTimestampFormat reference24 asDateAndTime.
	(format := ZTimestampFormat fromString: reference printString)
		createDateAndTime.
	self assert: (format format: reference) equals: reference printString.
	self assert: (format parse: reference printString) equals: reference.
	now := DateAndTime now truncated.
	self assert: (format format: now) equals: now printString.
	self assert: (format parse: now printString) equals: now.
	! !

!ZTimestampFormatTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 11/1/2012 20:39'!
testRfc3339
	| now |
	now := ZTimestamp now.
	self 
		assert: (ZTimestampFormat rfc3339 format: now)
		equals: now printString.
	self
		assert: (ZTimestampFormat rfc3339 parse: now printString)
		equals: now.
	self 
		assert: (ZTimestampFormat rfc3339 format: ZTimestampFormat reference12)
		equals: ZTimestampFormat reference12 printString.
	self
		assert: (ZTimestampFormat rfc3339 parse: ZTimestampFormat reference12 printString)
		equals: ZTimestampFormat reference12.
	self 
		assert: (ZTimestampFormat rfc3339 format: ZTimestampFormat reference24)
		equals: ZTimestampFormat reference24 printString.
	self
		assert: (ZTimestampFormat rfc3339 parse: ZTimestampFormat reference24 printString)
		equals: ZTimestampFormat reference24.! !

!ZTimestampFormatTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 11/1/2012 10:47'!
testTimeCreation
	self 
		assert: ((ZTimestampFormat fromString: '4:05:06PM') createTime; parse: '8:11:12PM')
		equals: (Time hour: 20 minute: 11 second: 12).	! !

!ZTimestampFormatTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 10/30/2012 14:43'!
testTimeFormatting
	self
		assert: ((ZTimestampFormat fromString: '04:05:06') 
						format: (ZTimestamp year: 0 month: 0 day: 0 hour: 11 minute: 30 second: 19))
		equals: '11:30:19'.! !

!ZTimestampFormatTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 10/31/2012 20:40'!
testTimeParsing
	self
		assert: ((ZTimestampFormat fromString: '04:05:06') parse: '11:30:19')
		equals: (ZTimestamp year: 0 month: 0 day: 0 hour: 11 minute: 30 second: 19).
	self
		assert: ((ZTimestampFormat fromString: '16:05:06') parse: '22:30:19')
		equals: (ZTimestamp year: 0 month: 0 day: 0 hour: 22 minute: 30 second: 19).
	self
		assert: ((ZTimestampFormat fromString: '04:05:06 PM') parse: '11:30:19 AM')
		equals: (ZTimestamp year: 0 month: 0 day: 0 hour: 11 minute: 30 second: 19).
	self
		assert: ((ZTimestampFormat fromString: '04:05:06 PM') parse: '11:30:19 PM')
		equals: (ZTimestamp year: 0 month: 0 day: 0 hour: 23 minute: 30 second: 19).! !

TestCase subclass: #ZTimestampTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ZTimestamp'!

!ZTimestampTests commentStamp: 'SvenVanCaekenberghe 4/20/2012 16:54' prior: 0!
I am ZTimestampTests, a test suite for ZnTimestamp.!

!ZTimestampTests methodsFor: 'asserting' stamp: 'SvenVanCaekenberghe 4/24/2012 17:36'!
assertInvariants: timestamp
	self assert: (timestamp isKindOf: ZTimestamp).
	self assert: timestamp julianDayNumber isInteger.
	self assert: (timestamp secondsSinceMidnight between: 0 and: (24 * 60 * 60) - 1).
	self assert: timestamp offset = Duration zero! !

!ZTimestampTests methodsFor: 'asserting' stamp: 'SvenVanCaekenberghe 4/21/2012 20:47'!
assertTimestamp: timestamp equalsTimestamp: reference 
	self assert: reference julianDayNumber = timestamp julianDayNumber.
	self assert: reference secondsSinceMidnight = timestamp secondsSinceMidnight.
	self assert: reference year = timestamp year.
	self assert: reference month = timestamp month.
	self assert: reference dayOfMonth = timestamp dayOfMonth.
	self assert: reference hour = timestamp hour.
	self assert: reference minute = timestamp minute.
	self assert: reference second = timestamp second! !

!ZTimestampTests methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/22/2012 10:20'!
lunarLanding
	"First lunar landing, Apollo 11, Mare Tranquillitatis"
	
	^ ZTimestamp fromString: '1969-07-20T20:17:40Z'! !

!ZTimestampTests methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/22/2012 10:20'!
marsLanding
	"First soft Mars landing, Viking 1, Chryse Planitia"
	
	^ ZTimestamp fromString: '1979-07-20T11:53:06Z'! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/21/2012 21:27'!
testAddingOneDay
	| t1 t2 |
	t1 := ZTimestamp year: 2012 month: 8 day: 7 hour: 14 minute: 20 second: 11.
	t2 := ZTimestamp year: 2012 month: 8 day: 8 hour: 14 minute: 20 second: 11.
	self assert: t1 < t2.
	self assert: (t1 + 1 day) = t2.! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/21/2012 21:28'!
testAddingOneHour
	| t1 t2 |
	t1 := ZTimestamp year: 2012 month: 8 day: 7 hour: 14 minute: 20 second: 11.
	t2 := ZTimestamp year: 2012 month: 8 day: 7 hour: 15 minute: 20 second: 11.
	self assert: t1 < t2.
	self assert: (t1 + 1 hour) = t2.! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/21/2012 20:48'!
testBasicFieldAccessors
	| timestamp |
	timestamp := ZTimestamp year: 2012 month: 4 day: 20 hour: 15 minute: 50 second: 10.
	self assert: timestamp year = 2012.
	self assert: timestamp month = 4.
	self assert: timestamp dayOfMonth = 20.
	self assert: timestamp hour = 15.
	self assert: timestamp minute = 50.
	self assert: timestamp seconds = 10	! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 7/6/2012 17:31'!
testBeginOfDay
	| timestamp |
	timestamp := ZTimestamp year: 2012 month: 4 day: 20 hour: 15 minute: 50 second: 10.
	self assert: timestamp asBeginOfDay equals: (ZTimestamp year: 2012 month: 4 day: 20)! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/22/2012 16:51'!
testCalculations
	| delta |
	delta := self marsLanding - self lunarLanding.
	self assert: delta > Duration zero.
	self assert: self lunarLanding + delta equals: self marsLanding.
	self assert: self marsLanding - delta equals: self lunarLanding! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/22/2012 10:51'!
testComparing
	self assertInvariants: self marsLanding.
	self deny: self lunarLanding = self marsLanding.
	self assert: self lunarLanding < self marsLanding.! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/3/2012 19:49'!
testConversions
	| timestamp |
	timestamp := ZTimestamp year: 2012 month: 4 day: 20 hour: 15 minute: 50 second: 10.
	self assert: timestamp asDate year equals: 2012.
	self assert: timestamp asDate monthIndex equals: 4.
	self assert: timestamp asDate dayOfMonth equals: 20.
	self 
		assert: timestamp asTime
		equals: (Time hour: 15 minute: 50 second: 10).
	self 
		assert: timestamp asTimeStamp 
		equals: (TimeStamp year: 2012 month: 4 day: 20 hour: 15 minute: 50 second: 10 offset: Duration zero).
	self 
		assert: timestamp asDateAndTime
		equals: (DateAndTime year: 2012 month: 4 day: 20 hour: 15 minute: 50 second: 10 offset: Duration zero).
	self assert: timestamp asZTimestamp = timestamp.
	self 
		assert: (TimeStamp year: 2012 month: 4 day: 20 hour: 15 minute: 50 second: 10 offset: Duration zero) asZTimestamp
		equals: timestamp.
	self 
		assert: (DateAndTime year: 2012 month: 4 day: 20 hour: 15 minute: 50 second: 10 offset: Duration zero) asZTimestamp
		equals: timestamp.
! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/22/2012 11:47'!
testCreationWithAllFields
	| reference timestamp |
	reference := DateAndTime 
		year: 2012 
		month: 4 
		day: 20 
		hour: 15 
		minute: 50 
		second: 10 
		nanoSecond: 0 
		offset: Duration zero.
	timestamp := ZTimestamp 
		year: reference year 
		month: reference month 
		day: reference dayOfMonth 
		hour: reference hour 
		minute: reference minute 
		second: reference second.
	self assertTimestamp: reference equalsTimestamp: timestamp! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/22/2012 11:47'!
testCreationWithDateFields
	| reference timestamp |
	reference := DateAndTime 
		year: 2012 
		month: 4 
		day: 20 
		hour: 0 
		minute: 0 
		second: 0 
		nanoSecond: 0 
		offset: Duration zero.
	timestamp := ZTimestamp 
		year: reference year 
		month: reference month 
		day: reference dayOfMonth.
	self assertTimestamp: reference equalsTimestamp: timestamp! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 7/6/2012 17:32'!
testEndOfDay
	| timestamp |
	timestamp := ZTimestamp year: 2012 month: 4 day: 20 hour: 15 minute: 50 second: 10.
	self assert: timestamp asEndOfDay equals: (ZTimestamp year: 2012 month: 4 day: 20 hour: 23 minute: 59 second: 59)! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/3/2012 19:19'!
testEpoch
	| epoch |
	epoch := ZTimestamp epoch.
	self assert: epoch equals: ZTimestamp zero.
	self assert: epoch julianDayNumber equals: 0.
	self assert: epoch secondsSinceMidnight equals: 0.
	"astronomical year to proleptic Gregorian year"
	self assert: epoch year equals: -4713.
	self assert: epoch month equals: 11.
	self assert: epoch dayOfMonth equals: 24.
	! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/21/2012 21:25'!
testEquality
	| t1 t2 |
	t1 := ZTimestamp year: 2012 month: 8 day: 7 hour: 14 minute: 20 second: 11.
	t2 := ZTimestamp year: 2012 month: 8 day: 7 hour: 14 minute: 20 second: 11.
	self assert: t1 = t2.
	self assert: ZTimestamp now = ZTimestamp now.
	self assert: ZTimestamp unixEpoch = ZTimestamp unixEpoch.! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/22/2012 11:21'!
testFormatting
	| timestamp formatter |
	timestamp := ZTimestamp year: 2012 month: 4 day: 20 hour: 15 minute: 50 second: 10.
	formatter := [ :format | 
		String streamContents: [ :stream | 
			timestamp printOn: stream format: format ] ].
	self assert: (formatter value: #($- $T $: $Z)) = '2012-04-20T15:50:10Z'.
	self assert: (formatter value: #(nil $T nil nil)) = '20120420T155010'.
	self assert: (formatter value: #($/ $| $: $Z)) = '2012/04/20|15:50:10Z'.
	self assert: (formatter value: #($- $T $: nil)) = '2012-04-20T15:50:10'.! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/22/2012 11:07'!
testHashing
	| array set now |
	now := ZTimestamp now.
	array := { now. self lunarLanding. self marsLanding. self unixTimeBillenium }.
	set := array asSet.
	self assert: array size = set size.
	array do: [ :each |
		self assert: (set includes: each) ].
	set addAll: array.
	self assert: array size = set size.
	array do: [ :each |
		self assert: (set includes: each) ].! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/22/2012 00:50'!
testJulianDate
	| now julianDate |
	now := ZTimestamp now.
	julianDate := now julianDate.
	self assert: (ZTimestamp fromJulianDate: julianDate) - now < 1 second! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/3/2012 16:36'!
testLocalPrintString
	| timestamp reference |
	timestamp := ZTimestamp now localPrintString.
	reference := DateAndTime now truncated printString.
	self assert: timestamp equals: reference! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/22/2012 00:45'!
testLunarLanding
	| timestamp |
	timestamp := self lunarLanding.
	self assertInvariants: timestamp.
	self assert: timestamp monthName = #July.
	self assert: timestamp dayOfWeekName = #Sunday.
	self assert: timestamp dayOfYear = 201.! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/22/2012 19:05'!
testMixedCalculations
	| now delta |
	now := DateAndTime now truncated asUTC.
	delta := now - self lunarLanding.
	self assert: delta > Duration zero.
	self assert: self lunarLanding + delta equals: now asZTimestamp.
	self assert: (self lunarLanding + delta) asDateAndTime equals: now.
	self assert: (now - delta) asZTimestamp equals: self lunarLanding.
	self assert: (now - delta) equals: self lunarLanding asDateAndTime! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/21/2012 20:40'!
testNow
	self assertInvariants: ZTimestamp now! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/21/2012 21:28'!
testOneDayDifference
	| t1 t2 |
	t1 := ZTimestamp year: 2012 month: 8 day: 7 hour: 14 minute: 20 second: 11.
	t2 := ZTimestamp year: 2012 month: 8 day: 8 hour: 14 minute: 20 second: 11.
	self assert: t1 < t2.
	self assert: (t2 - t1) = 1 day! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/21/2012 21:28'!
testOneHourDifference
	| t1 t2 |
	t1 := ZTimestamp year: 2012 month: 8 day: 7 hour: 14 minute: 20 second: 11.
	t2 := ZTimestamp year: 2012 month: 8 day: 7 hour: 15 minute: 20 second: 11.
	self assert: t1 < t2.
	self assert: (t2 - t1) = 1 hour.! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 7/6/2012 17:14'!
testParsing
	| timestamp |
	timestamp := ZTimestamp year: 2012 month: 8 day: 7 hour: 14 minute: 20 second: 11.
	self assert: (ZTimestamp readFrom: '2012-08-07T14:20:11Z' readStream) = timestamp.
	self assert: (ZTimestamp readFrom: '20120807T142011' readStream) = timestamp.
	self assert: (ZTimestamp readFrom: '20120807142011' readStream) = timestamp.
	self assert: (ZTimestamp readFrom: '2012-08-07T14:20:11' readStream) = timestamp.
	self assert: (ZTimestamp readFrom: '2012-08-07T14:20:11+00:00' readStream) = timestamp.
	self assert: (ZTimestamp readFrom: '2012-08-07T16:20:11+02:00' readStream) = timestamp.
	self assert: (ZTimestamp readFrom: '2012-08-07T12:20:11-02:00' readStream) = timestamp.! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/21/2012 22:14'!
testParsingDates
	| timestamp |
	timestamp := ZTimestamp year: 2012 month: 8 day: 7.
	self assert: (ZTimestamp readFrom: '2012-08-07' readStream) = timestamp.
	self assert: (ZTimestamp readFrom: '20120807' readStream) = timestamp.! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/20/2012 21:35'!
testPrintString
	| timestamp |
	timestamp := ZTimestamp year: 2012 month: 4 day: 20 hour: 15 minute: 50 second: 10.
	self assert: timestamp printString = '2012-04-20T15:50:10Z'! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/23/2012 08:27'!
testPrintStringCompatibility
	| dateTime timestamp |
	dateTime := DateAndTime now truncated printString.
	timestamp := (ZTimestamp fromString: dateTime) localPrintString.
	self assert: dateTime equals: timestamp! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/28/2012 09:28'!
testSTON
	Smalltalk at: #STON ifPresent: [ :ston | 
		| now |
		now := ZTimestamp now.
		self 
			assert: (ston fromString: (ston toString: now)) 
			equals: now ]! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/22/2012 11:03'!
testSorting
	| array sorted now |
	now := ZTimestamp now.
	array := { now. self lunarLanding. self marsLanding. self unixTimeBillenium }.
	sorted := array sorted.
	self assert: sorted first = self lunarLanding.
	self assert: sorted last = now! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/22/2012 11:35'!
testTimezoneFormatting
	| timestamp formatter |
	timestamp := ZTimestamp year: 2012 month: 4 day: 20 hour: 15 minute: 50 second: 10.
	formatter := [ :format :timezone| 
		String streamContents: [ :stream | 
			timestamp printOn: stream format: format timezone: timezone] ].
	self assert: (formatter value: #($- $T $: nil) value: 2 hours) = '2012-04-20T17:50:10+02:00'.
	self assert: (formatter value: #($- $T $: nil) value: -2 hours) = '2012-04-20T13:50:10-02:00'.
	self assert: (formatter value: #(nil $T nil nil) value: 1 hours) = '20120420T165010+0100'.
	self assert: (formatter value: #(nil $T nil nil) value: -1 hours) = '20120420T145010-0100'.
! !

!ZTimestampTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/22/2012 11:03'!
testUnixTime
	| timestamp billenium |
	timestamp := ZTimestamp fromUnixTime: 1000000000.
	billenium := self unixTimeBillenium.
	self assert: timestamp = billenium.
	self assert: billenium asUnixTime = 1000000000! !

!ZTimestampTests methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/22/2012 11:02'!
unixTimeBillenium
	"1 billion seconds since the Unix Epoch"
	
	^ ZTimestamp fromString: '2001-09-09T01:46:40Z'.! !
ZTimestamp initialize!
ZTimestampFormat initialize!
